/*******************************************************************************************
 *                                                                                         *
 *----------- multi-knapsack problem described in Zitzler's PhD Dissertation --------------*
 *                                                                                         *
 *                                                          Xianming Chen, Feb 23, 2001     *
 *-------------------------- see the Makefile for usage -----------------------------------*
 *******************************************************************************************/


#include <moea/NPGA.h>
#include "knapsack.h"

void  main(int argc, char** argv) {

  NPGA npga;  npga.maximize();

  int items = 250;
  npga.addParameter("items of each knapsack", "items", "items", Int, &items);

  npga.readCommandLine(argc, argv);  

  initProblem(items, npga.objectiveDimensions());

  IND ind(items, knapsackEvaluator);
  Normalizer objN;
  for(int i=0; i<ind.objectiveDimensions(); i++)
    objN.add(0, maxProfitVector[i]);
  ind.objectiveNormalizer(objN);
  ind.metric(DefaultNormalizedObjectiveDistance);

  npga.initialize(ind);
  startLog(npga);

  while( !npga.done() ) { 
    if(npga.currentGeneration() % 100 == 0) {
      npga.parameters();
      npga.printScores();
    }
    npga.step();
  }
  npga.parameters();
  npga.printScores();
  endLog(npga);
}


